const ObjectID    = require("mongodb").ObjectID;
const {connect}   = require('mm')

module.exports = async (dbName='amazon2home') => {
    const {Savable, slice} = await connect(dbName)

    async function getModels({id}){
        const SlicedSavable = slice([id, 'user'])

        class User extends SlicedSavable {
            constructor(...params){
                super(...params)
                //TODO: calc likes count by getter (no two-way relation for this to avoid overflow on many Kilos of likes
                //cached like count, which incremented and decremented
                //
                //following and followers array

            }


            static get relations(){ //don't needed due to ___owner in most cases
                return {
                    avatar : "userAvatar",
                    executedOrders: "executor"
                }
            }
        }
        SlicedSavable.addClass(User)

        class OwnerSlicedSavable extends SlicedSavable {
            get owner(){
                if (!this.___owner) return this.___owner

                return SlicedSavable.m.User.findOne({_id: ObjectID(this.___owner)})
            }
        }


        class Image extends OwnerSlicedSavable {
            constructor(...params){
                super(...params)
            }


            static async fromFileData(fileData){
                let image  = new Image({})
                image.fileData = fileData
                image.url      = `images/${fileData.filename}`
                image.originalFileName = fileData.originalname
                await image.save()
                return image;
            }

            async save(...params){
                if (this.userAvatar){
                    if (this.userAvatar._id.toString() !== id){
                        throw new ReferenceError(`You can't set ava for other user`)
                    }
                }

                return await super.save(...params)
            }

            static get relations(){
                return {
                    userAvatar: "avatar", //if it is ava
                    order: ["images"] 
                }
            }

        }
        SlicedSavable.addClass(Image)

        class Order extends OwnerSlicedSavable {
            constructor(...params){
                super(...params)
            }

            static get relations(){
                return {
                    executor: ["executedOrders"],
                    messages: "order",
                    comments: "order",
                    images: "order"
                }
            }
        }
        SlicedSavable.addClass(Order)

        class Message extends OwnerSlicedSavable {
            constructor(...params){
                super(...params)
            }

            static get relations(){
                return {
                    order: ["messages"]
                }
            }
        }
        SlicedSavable.addClass(Message)


        class Comment extends OwnerSlicedSavable {
            constructor(...params){
                super(...params)
            }

            static get relations(){
                return {
                    order: ["comments"]
                }
            }
        }
        SlicedSavable.addClass(Comment)









        const thisUser = await Savable.m.User.findOne({_id: ObjectID(id)})

        return {models: {
                            SlicedSavable, ...SlicedSavable.classes
                        }, 
                thisUser}
    }

    return {
        Savable, 
        slice,
        getModels
    }
}


